local M = {}

local ct    = nil
local c     = nil
local paint = {}

local bestLapStunt = 0
local bestLapSpeed = 0
local lastLapStunt = 0
local lastLapSpeed = 0
local lastSectorStunt = 0
local lastSectorSpeed = 0
local sectorTimesStunt = {}
local sectorTimesSpeed = {}
local lapCounter = 1
local timeLeft = 80
local score = 0
local highScore = 0
local highScoreLaps = 0

--[[ gets current vehicle directory ]]--
local function loadvName()
	dofile( "lua/theNostalgiaIILT/table.save-1.0.lua" )
	local vName, err = table.load( "lua/vName.data" )
	if err == nil then
		return vName["vehicleDirectory"]
		
	else
		print("Error loading map: "..err)
		return {}
	end
end

--[[ gets current vehicle name from name.cs file ]]--
local function loadVehicleName(vDirectory)
	local vName = loadvName()
	if vName ~= nil then
		local file = io.open(vName .. "name.cs", "r") -- open file in read only
		contents = file:read()
		file:close()
		vehicleName = contents
		local b = BeamEngine:getSlot(0)
		b:queueLuaCommand("drivetrain.fuel = 40")
		b:queueLuaCommand("Settings.gravity = -10.1")
		-- b:queueLuaCommand("input.event(\'axisy2\', .2, 2)")
	else
		vehicleName = "N/A"
	end
end
loadVehicleName("vehicleDirectory")
--[[ ]]--

--[[ converts seconds to xx:xx:xx format ]]--
local function timeCount(numSec)
	local nSeconds = numSec
		if nSeconds == 0 then

				coolTime = "00:00:00";
				return coolTime;
		else
			local	nHours = string.format("%02.f", math.floor(nSeconds/3600));
			local	nMins = string.format("%02.f", math.floor(nSeconds/60 - (nHours*60)));
			local	nSecs = string.format("%02.f", math.floor(nSeconds - nHours*3600 - nMins *60));
			local	nHSecs = string.format("%.3f", nSeconds);
			local	hunSecs = string.sub(nHSecs, -3, -2);
				-- coolTime =  nHours..":"..nMins..":"..nSecs
				coolTime =  nMins..":"..nSecs..":"..hunSecs
				return coolTime;
	end
end
--[[ ]]--

--[[ font setup ]]--
local function initPaint()
    local fontName = "lua/theNostalgiaIILT/BebasNeue.otf"
    local tf = SkTypeface.CreateFromFile(fontName, kNormal)
    
	--[[ drop shadows for text visibility ]]--
	paint.fontNormalShadow = SkPaint()
	paint.fontNormalShadow:setColor(0xFFFF0000)
    paint.fontNormalShadow:setAntiAlias(true)
    paint.fontNormalShadow:setARGB(200,0,0,0)
	paint.fontNormalShadow:setTextAlign(kLeft_Align)
    paint.fontNormalShadow:setTextSize(30)
    paint.fontNormalShadow:setTypeface(tf)
	
	paint.fontSmallShadow = SkPaint()
    paint.fontSmallShadow:setAntiAlias(true)
    paint.fontSmallShadow:setARGB(200,0,0,0)
	paint.fontSmallShadow:setTextAlign(kLeft_Align)
    paint.fontSmallShadow:setTextSize(24)
    paint.fontSmallShadow:setTypeface(tf)
	
	paint.fontTimeShadow = SkPaint()
    paint.fontTimeShadow:setAntiAlias(true)
    paint.fontTimeShadow:setARGB(200,0,0,0)
	paint.fontTimeShadow:setTextAlign(kLeft_Align)
    paint.fontTimeShadow:setTextSize(36)
    paint.fontTimeShadow:setTypeface(tf)
	
	--[[ text ]]--
    paint.fontNormal = SkPaint()
	paint.fontNormal:setColor(0xFFFF0000)
    paint.fontNormal:setAntiAlias(true)
    paint.fontNormal:setARGB(244,255,255,255)
	paint.fontNormal:setTextAlign(kLeft_Align)
    paint.fontNormal:setTextSize(30)
    paint.fontNormal:setTypeface(tf)
		
	paint.fontSmall = SkPaint()
    paint.fontSmall:setAntiAlias(true)
    paint.fontSmall:setARGB(244,255,255,255)
	paint.fontSmall:setTextAlign(kLeft_Align)
    paint.fontSmall:setTextSize(24)
    paint.fontSmall:setTypeface(tf)
	
	paint.fontTime = SkPaint()
    paint.fontTime:setAntiAlias(true)
    paint.fontTime:setARGB(222,255,255,255)
	paint.fontTime:setTextAlign(kLeft_Align)
    paint.fontTime:setTextSize(36)
    paint.fontTime:setTypeface(tf)
end
--[[ ]]--

--[[ main display updated once per second ]]--
local function printTimesSpeed(CP)
    if not ct then
        ct = BeamEngine.canvasTexture
    end
    c = ct:getCanvas()
    ct:clear()
    
    if ( paint.fontNormal == nil ) then
        initPaint()
    end
		
	bestSpeed = timeCount(bestLapSpeed)
	lastSpeed = timeCount(lastLapSpeed)
	lsSpeed = timeCount(lastSectorSpeed)
	timeLeft2 = string.sub(timeCount(timeLeft), 2, -4)
	
	vNameText = string.sub(vehicleName, 17, -3)	
	vNameTextS = string.format("%s", vNameText)
	lapSpeedText = string.format("BEST: %s | LAST: %s | CURRENT: %s", bestSpeed, lastSpeed, lsSpeed);
	highScoreText = string.format("BEST: %s - LAP %s | CURRENT: %s - LAP %s", highScore, highScoreLaps, score, lapCounter);
	mphText = string.format("%s MPH", mph)
	
	--[[ drop shadows ]]--
	c:drawText(vNameTextS, string.len(vNameTextS), 21, 20, paint.fontSmallShadow)
	c:drawText(lapSpeedText, string.len(lapSpeedText), 21, 44, paint.fontSmallShadow)
	c:drawText(highScoreText, string.len(highScoreText), 21, 68, paint.fontSmallShadow)
	c:drawText(timeLeftTxt, string.len(timeLeftTxt), 21, 104, paint.fontTimeShadow)
	c:drawText(mphText, string.len(mphText), 19, 144, paint.fontTimeShadow)
	
	c:drawText(vNameTextS, string.len(vNameTextS), 19, 20, paint.fontSmallShadow)
	c:drawText(lapSpeedText, string.len(lapSpeedText), 19, 44, paint.fontSmallShadow)
	c:drawText(highScoreText, string.len(highScoreText), 19, 68, paint.fontSmallShadow)
	c:drawText(timeLeftTxt, string.len(timeLeftTxt), 19, 104, paint.fontTimeShadow)
	c:drawText(mphText, string.len(mphText), 19, 144, paint.fontTimeShadow)
	
	--[[ ]]--
	c:drawText(vNameTextS, string.len(vNameTextS), 20, 20, paint.fontSmall)
    c:drawText(lapSpeedText, string.len(lapSpeedText), 20, 44, paint.fontSmall)
	c:drawText(highScoreText, string.len(highScoreText), 20, 68, paint.fontSmall)
	c:drawText(timeLeftTxt, string.len(timeLeftTxt), 20, 104, paint.fontTime)
	c:drawText(mphText, string.len(mphText), 20, 144, paint.fontTime)
end

--[[ ]]--
local function printTimesBest(started)
   --[[ converts time to xx:xx:xx ]]--
	bestSpeed = timeCount(bestLapSpeed)
	lastSpeed = timeCount(lastLapSpeed)

	
	--[[ formats into a string ]]--
	bestLapTxtSpeed = string.format("Best: %s", bestSpeed);
	lastLapTxtSpeed = string.format("Last: %s", lastSpeed);
	
	
	vNameText = string.sub(vehicleName, 17, -3)
	vNameTextS = string.format("Times for: %s", vNameText)
end

--[[ things to do when lap is completed ]]--
local function lapComplete(total, trackName)
	if trackName == "RDSST" then
		for i=1, total-1 do
			if sectorTimesSpeed[i] == nil or sectorTimesSpeed[i+1] == nil then
				return false
			elseif sectorTimesSpeed[i] > sectorTimesSpeed[i+1] then
				return false
			end
		end
	end
	
	lapCounter = lapCounter + 1
	return true
end
--[[ ]]--

--[[ load / save high score ]]--
local function loadHighScore()
	dofile( "lua/theNostalgiaIILT/table.save-1.0.lua" )
	local highScores, err = table.load( "lua/theNostalgiaIILT/theNostalgiaIIScores.data" )
	if err == nil then
		return highScores
	else
		print("Error loading map: "..err)
		return {}
	end
end
local function loadHighScores()
	local highScores = loadHighScore()

	--[[ gets high score ]]--
	if highScores[1] ~= nil then
		highScore = highScores[1]
		highScoreLaps = highScores[2]
	else
		highScore = 0
		highScoreLaps = 0
	end
end
local function saveHighScore(HS, lapCount)
	dofile( "lua/theNostalgiaIILT/table.save-1.0.lua" )
	highScores = loadHighScore()
	highScores = { [1] = HS, [2] = lapCount }
	table.save( highScores, "lua/theNostalgiaIILT/theNostalgiaIIScores.data" )
end
--[[ ]]--

--[[ load / save best lap times ]]--
local function loadSpeedTimes()
	dofile( "lua/theNostalgiaIILT/table.save-1.0.lua" )
	local bestSpeedLaps, err = table.load( "lua/theNostalgiaIILT/theNostalgiaIITimes.data" )
	if err == nil then
		return bestSpeedLaps
	else
		print("Error loading map: "..err)
		return {}
	end
end
local function loadTimeBoth()
	local bestSpeedLaps = loadSpeedTimes()
	loadVehicleName("vehicleDirectory")
	vNameText = string.sub(vehicleName, 17, -3)	
	
	--[[ gets best lap time for current vehicle ]]--
	for _,v in pairs(bestSpeedLaps) do
		if _ == vNameText then
			if bestSpeedLaps[_][1] == "RDSST" then
				if bestSpeedLaps[_][1] ~= nil then
					bestLapSpeed = bestSpeedLaps[_][2]
				else
					bestLapSpeed = 0
				end
			end
		break
		else
			bestLapSpeed = 0
		end
	end
end
local function saveTime(levelname, time)
	dofile( "lua/theNostalgiaIILT/table.save-1.0.lua" )
	
	if levelname == "RDSST" then
		bestSpeedLaps = loadSpeedTimes()
		bestSpeedLaps[vNameText] = { [1] = levelname, [2] = time }
		table.save( bestSpeedLaps, "lua/theNostalgiaIILT/theNostalgiaIITimes.data" )
	end
end
--[[ ]]--

--[[ update routine triggered by checkpoints ]]--
local function update(sector, total, levelname, cptype)
    local currentTime = os.clock()
	if levelname == "RDSST" then
		if lapComplete(total, "RDSST") then
			lastLapSpeed = currentTime-sectorTimesSpeed[1]
			lastSectorSpeed = 0
			if bestLapSpeed == 0 or lastLapSpeed < bestLapSpeed then
				bestLapSpeed = lastLapSpeed
				
				if levelname ~= nil then
					saveTime(levelname, bestLapSpeed)
				end
			end
		end
		
		sectorTimesSpeed[sector] = currentTime
		if sectorTimesSpeed[1] ~= nil then
			lastSectorSpeed = sectorTimesSpeed[sector]-sectorTimesSpeed[1]
		end

		--normal start
		if ( lapCounter == 1 and cptype == "START" ) then
			timeLeft = 80
			score = 0
		end
		
		--cheap start
		if ( lapCounter == 1 and cptype == "START" and mph > 60 ) then
			timeLeft = 0
			score = 0
			disqualified = "YES"
		end
		
		if ( cptype == "NORMAL" ) then
			timeLeft = timeLeft + 30
			addTime = "NORMAL"
		end
		
		if ( cptype == "FINISH" ) then
			timeLeft = timeLeft + 45
			addTime = "FINISH"
		end
	
		if ( lapCounter > 1 and cptype == "START" ) then
			timeLeft = timeLeft
		end
		
		-- os.execute("D:\\Users\\Mike\\AppData\\Local\\BeamNG\\BeamNG.drive - Copy\\lua\\theNostalgiaIILT\\checkPoint.exe")
	end
end

--[[ timer handler ]]--
local function checkTime()
	if ( disqualified == "YES" ) then
		local b = BeamEngine:getSlot(0)
		b:queueLuaCommand("drivetrain.fuel = 0")
		timeLeftTxt = string.format("D I S Q U A L I F I E D");
		local b = BeamEngine:getSlot(0)
		b:queueLuaCommand("input.event(\'axisy2\', .15, 2)")
		mph = "COASTING"
	elseif ( timeLeft < 1 ) then
		local b = BeamEngine:getSlot(0)
		b:queueLuaCommand("drivetrain.fuel = 0")
		-- timeLeft = 0
		timeLeftTxt = string.format("G A M E  O V E R");
		if highScore == 0 or score > highScore then
			highScore = score
			saveHighScore(highScore, lapCounter)
		end
		local b = BeamEngine:getSlot(0)
		b:queueLuaCommand("input.event(\'axisy2\', .15, 2)")
		-- b:queueLuaCommand("beamstate.deflateTire(0)")
		mph = "COASTING"
	else
		timeLeft = timeLeft - 1
		local b = BeamEngine:getSlot(0)
		b:queueLuaCommand("drivetrain.fuel = " .. timeLeft)
		b:queueLuaCommand("input.event(\'axisy2\', 0, 2)")
		airSpeed	= b:getVelocity():length()
		kmh			= math.floor( airSpeed * 3.6 )
		mph			= math.floor( kmh * 0.6214 )
		if ( addTime == "NORMAL" ) then
			timeLeftTxt = string.format("Time Left: %s +30sec", timeLeft2);
			addTime = "RESET"
		elseif ( addTime == "FINISH" ) then
			timeLeftTxt = string.format("Time Left: %s +45sec", timeLeft2);
			addTime = "RESET"
		else
			timeLeftTxt = string.format("Time Left: %s", timeLeft2);
		end
		score = score + mph * 7
		loadHighScores()
		loadTimeBoth("RDSST")
		highScoreText = string.format("BEST: %s ( %s LAP(S) ) | CURRENT: %s", highScore, highScoreLaps, score);
	end	
end


--[[ speedometer 
local function speedometer()
	
	--	Calculate car speed
	local car			= BeamEngine:getSlot( 0 )
	local airSpeed		= car:getVelocity():length()
	local kmh			= math.floor( airSpeed * 3.6 )
	local mph			= math.floor( kmh * 0.6214 )
	
	--	Return car speeds
	--  return mph
  	--	Gauge border brush
	local brushes			= {}
	local brush			= SkPaint()
	brush:setAntiAlias( true )
	brush:setARGB( 255, 0, 0, 0 )
	brushes.gaugeBorder	= brush
	local rect			= createSKRect( x, y, w, h )
	local sRect			= createSKRect( x - 2, y - 2, w + 4, h + 4 )
	c:drawRect( sRect, brushes.gaugeBorder )
	c:drawText(mph, string.len(mph), 20, 170, paint.fontNormal)
end
]]--

M.update    = update
M.checkTime    = checkTime
M.printTimesStunt  = printTimesStunt 
M.printTimesSpeed  = printTimesSpeed
M.loadVehicleName = loadVehicleName
M.loadTimeBoth = loadTimeBoth
M.loadHighScores = loadHighScores
M.printTimesBest  = printTimesBest
M.changeGravity = changeGravity

return M